cspe	All comments indicated with a cspe by Steven Esling for GraphicKGS.xls
cspe  The code was modifed to run as a dynamic link library under Microsoft Windows
cspe  and attribute statements were also added to subroutines called from Microsoft Excel Visual Basic
cspe
C     Last change:  JJB  31 Jul 99    2:16 pm
C******************************************************************************
C                             KGSMOD.FOR
C       PROGRAM TO EVALUATE SEMIANALYTICAL SOLUTION FOR CASE OF SLUG TESTS
C       IN CONFINED OR UNCONFINED AQUIFERS IN PARTIALLY PENETRATING WELLS 
C       WITH OR WITHOUT A FINITE-RADIUS WELL SKIN.  SOLUTION EVALUATED
C       FOR HEAD AT THE TEST WELL OR AT A NEARBY OBSERVATION WELL.  FOURIER 
C       AND LAPLACE INVERSIONS DONE NUMERICALLY USING A TRUNCATED INFINITE-
C       SERIES SUMMATION AND THE STEHFEST ALGORITHM,RESPECTIVELY.  EMPLOYS THE
C       FOLLOWING CONDITION FOR THE FOURIER INVERSION: HEAD IN THE TEST WELL 
C       OR AT AN OBSERVATION WELL IS THE VERTICAL AVERAGE OF HEADS IN THE 
C       FORMATION ALONG THE WELL SCREEN. NOTE THAT EARLIER VERSIONS OF THIS 
C       PROGRAM WERE NAMED CONSLUG.FOR (V.1.1) AND KGSMOD2.FOR (V.2.0).        
C
C       DATE OF ORIGINAL PROGRAM: FEBRUARY, 1990            
C       PROGRAMMER: W.LIU           
C       FIRST REVISION (V.1.1): JANUARY, 1994 - COMMENTS AND MODIFICATIONS
C             ADDED BY J.BUTLER.
C       SECOND REVISION (V. 2.0): FEBRUARY, 1995 - UNCONFINED OPTION, NEW
C             WELL-BORE BOUNDARY CONDITION, AND COMMENTS ADDED BY J.BUTLER.
C       THIRD REVISION (V. 3.0): JULY, 1995 - OBSERVATION-WELL OPTION AND 
C             COMMENTS ADDED BY J.BUTLER.
C       CURRENT VERSION: 3.0
C                                              
C       SEE HYDER, BUTLER, MCELWEE, AND LIU, SLUG TESTS IN PARTIALLY      
C          PENETRATING WELLS, WATER RESOURCES RESEARCH, V. 30, NO. 11,
C          PP. 2945-2957, NOV., 1994 FOR DERIVATION AND EXPLANATION
C          OF EQUATIONS MENTIONED IN PROGRAM COMMENTS.  
C                                            
C       SEE BUTLER, SLUG TESTS WITH OBSERVATION WELLS: EXTENSION OF HYDER ET 
C          AL. SOLUTION TO CASE OF HEAD IN AN OBSERVATION WELL, KGS OPEN-FILE
C          REPT. 95-43, 14 PP., 1995 FOR DERIVATION AND EXPLANATION OF 
C          EQUATIONS MENTIONED IN PROGRAM COMMENTS FOR OBSERVATION-WELL OPTION.
C
C       RS -- WELL SCREEN RADIUS
C       RC -- WELL CASING RADIUS
C       RK -- WELL SKIN RADIUS
C       S1 AND S2 -- SPECIFIC STORAGE OF SKIN AND AQUIFER, RESPECTIVELY.
C       KR1 AND KR2 -- RADIAL CONDUCTIVITY OF SKIN AND AQUIFER, RESPECTIVELY.
C       KZ1 AND KZ2 -- VERTICAL CONDUCTIVITY OF SKIN AND AQUIFER, RESPECTIVELY.
C       H0 -- INITIAL HEAD (SIZE OF INITIAL DISPLACEMENT - SIZE OF SLUG)
C       ROBS -- RADIAL LOCATION OF OBSERVATION POINT
C       Z1 AND Z2 -- DEPTH FROM TOP OF AQUIFER TO TOP AND BOTTOM, RESPECTIVELY,
C          OF THE OBSERVATION-WELL SCREEN
C       B -- AQUIFER THICKNESS
C       B1 -- DEPTH FROM TOP OF THE AQUIFER (WATER TABLE IN UNCONFINED 
C          SYSTEM) TO THE TOP OF THE SCREEN 
C       B2 -- SCREEN LENGTH
C       NQZ -- NO. OF STEHFEST WEIGHTING FACTORS
C       RERR -- MAXIMUM ALLOWABLE RELATIVE ERROR - IF THE RELATIVE DIFFERENCE
C          BETWEEN TWO CONSECUTIVE HEADS CALCULATED BY THE FOURIER SERIES
C          SUMMATION AND THE STEHFEST ALGORITHM IS LESS THAN RERR, THEN
C          THE NUMERICAL INVERSION IS COMPLETED FOR THE CURRENT TIME.
C       ICN -- SET TO ZERO IF SCREEN IS AT THE CENTER OF A CONFINED AQUIFER
C       TAU -- INITIAL TIME 
C       NTAU -- NO. OF TIME STEPS
C       TAUINC -- TIME STEP ACCELERATION FACTOR
C       ISKIN -- INDICATOR FOR NO-SKIN (=0) OR SKIN (=1) CASE
C       IFLAG -- INDICATOR FOR CONFINED (=1) OR UNCONFINED (=2) CASE
C       IOBS -- INDICATOR FOR CASE OF HEADS AT THE TEST WELL (=0) OR AT AN 
C          OBSERVATION WELL (=1) 
C       HTOL -- MINIMUM ALLOWABLE NORMALIZED HEAD, IF NORMALIZED HEAD IS
C          LESS THAN HTOL AND HEADS ARE DECREASING, THEN EXIT PROGRAM.
C
C  ********************************************************************
C
C   Note:
C
C       1.  File "KGSMOD.RES" contains real time-drawdown solution
C       2.  File "DIM.RES" contains the dimensionless time-normalized
C          drawdown solution
C**********************************************************************
C
cspe  This routine was reproduced as a Visual Basic subroutine in GraphicKGS.xls. This main routine is not called from 
cspe  GraphicKGS.xls.  One change was made, however.  The call to FSUM was commented out because of changes in the FSUM subroutine
cspe  declaration statement.  Without this change, the code would not compile. 
C
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION FP(20), ASUM1(20), ASUM2(20)
      DOUBLE PRECISION KR1, KZ1, KR2, KZ2, LAMBDA
      COMMON /CMPPSLUG/ RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,
     *                  X1,X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2
      OPEN (3,FILE='KGSMOD.DAT',STATUS='UNKNOWN')
      OPEN (8,FILE='DIM.RES',STATUS='UNKNOWN')
      OPEN (9,FILE='KGSMOD.RES',STATUS='UNKNOWN')
      OPEN (12,FILE='ERRORMSG.RES',STATUS='UNKNOWN')
      PI = 3.1415926535897932
C
C  Read in data from file "KGSMOD.DAT"
C
      READ (3,*) IFLAG
      IF (IFLAG.LT.1.OR.IFLAG.GT.2) THEN
          WRITE(12,*) 'IFLAG INCORRECT - PLEASE CHANGE'
          GOTO 9999
      ENDIF
      READ (3,*) RS, RC, S2, Kr2, Kz2, HO 
      READ (3,*) B, B1, B2, RERR, NQZ,ICN,HTOL
      READ (3,*) TAU, NTAU, TAUINC
      WRITE (12,*) RS, RC, S2, Kr2, Kz2, HO
      WRITE (12,*) B, B1, B2, RERR, NQZ,ICN,HTOL
      WRITE (12,*) TAU, NTAU, TAUINC
CC
C   READ IN WHETHER THERE IS A WELL SKIN
C
      READ (3,*) ISKIN                       
      WRITE (12,*) ISKIN
      IF (ISKIN.LT.0.OR.ISKIN.GT.1) THEN
          WRITE(12,*) 'ISKIN INCORRECT - PLEASE CHANGE'
          GOTO 9999
      ENDIF    
      IF(ISKIN.EQ.0) THEN 
         READ(3,*)
         S1=S2
         KR1=KR2
         KZ1=KZ2
         RK = RS
      ELSE
C   READ IN SKIN PARAMETERS S1, Kr1, Kz1,RK
         READ (3,*) S1, Kr1, Kz1, RK
         WRITE (12,*) S1, Kr1, Kz1, RK
      ENDIF
C
C   READ IN WHETHER OBS. WELL HEADS ARE TO BE CALCULATED
C
      READ (3,*) IOBS                       
      WRITE (12,*) IOBS
      IF (IOBS.LT.0.OR.IOBS.GT.1) THEN
          WRITE(12,*) 'IOBS INCORRECT - PLEASE CHANGE'
          GOTO 9999
      ENDIF    
      IF(IOBS.EQ.1) THEN
C
C   READ IN OBSERVATION WELL LOCATION
C
         READ (3,*) ROBS, Z1, Z2
         WRITE (12,*) ROBS, Z1, Z2
C
C   CALCULATE OBS. WELL PARAMETERS
C 
         R = ROBS/RS
         ZD = (Z2 - Z1)/B2
         Z12 = (Z2 + Z1)/B2         
      ENDIF
C
C  NOW CALCULATE PARAMETERS REPEATEDLY USED IN SUBSEQUENT LOOPS
C                                                       
C  CALCULATE THE DIMENSIONLESS FORMATION THICKNESS
C
      BETA = B/B2
C      
C    CALCULATE GAMMA2 PARAMETER OF EQNS. (A14) AND (A18)
C                                                       
      GAMMA2 = KR2/(KR1*2.0)
C
C     CALCULATE R1 AND R2 TERMS USED IN NU PARAMETER OF EQN. (A11)
C
      ALPHA = (2.0*RS*RS*S2*B2)/(RC*RC)
      LAMBDA = S2/S1
      R1 = (GAMMA2*ALPHA)/LAMBDA
      R2 = ALPHA/2.0
C
C      CALCULATE NORMALIZED SKIN RADIUS
C
      RNK = RK/RS              
      IF (RNK.LT.1.) THEN
          WRITE(12,*) 'SKIN RADIUS INCORRECT - PLEASE CHANGE'
          GO TO 9999
      ENDIF
C
C   CALCULATE COEFS. USED IN SINE AND COSINE TERMS IN FOURIER
C      BACK TRANSFORM IN EQNS. (B6) AND (B7)
C
      IF(IFLAG.EQ.2)THEN
         B=2.0D0*B
      ENDIF
      X1 = PI * B2 / (2.0 * B)
      X2 = PI * (2.0*B1 + B2) / (2.0 * B)
C
C   CALCULATE COEFS. USED IN SINE AND COSINE TERMS IN FOURIER
C      BACK TRANSFORM IN EQNS. (17) AND (25) OF BUTLER (1995)
C
      IF(IOBS.EQ.1) THEN
         Z1 = X1*ZD
         Z2 = X1*Z12
      ENDIF         
C   
C   CALCULATE COEFS. IN NU TERMS OF EQN. (A11)
C                 
      PI2 = PI*PI
      AST1 = (KZ1*PI2*RS*RS)/(KR1*B*B)
      AST2 = (KZ2*PI2*RS*RS)/(KR2*B*B)
C
C    MULTIPLY FLOW PROPERTIES BY SCREEN LENGTH
C
      T1 = Kr1 * B2
      T2 = Kr2 * B2
      S1 = S1 * B2
      S2 = S2 * B2
C 
C  CALCULATE NO. OF TERMS TO BE SUMMED EACH TIME THE FSUM
C  SUBROUTINE IS CALLED. NOTE: NNMM IS CALCULATED SO THAT THE
C  SUMMATION IS PERFORMED FOR AT LEAST 4 CYCLES OF SIN(NQ*X1).
C  MAXIMUM NO. OF TERMS THAT WILL BE SUMMED AT ONE TIME BY
C  FSUM IS 50.
C
      NNMM = 8 * INT ((2*B) / B2 )
      IF(NNMM .GT. 50) NNMM = 50
      IF (TAU .LT. 0.0) THEN
         WRITE(12,*) ' TAU IS NEGATIVE - PLEASE CORRECT'
         GOTO 9999
      ENDIF
      N = NQZ      
C
C  DEFINE INITIAL TIME STEP AND CONVERT REAL TIME TO THE DIMENSIONLESS
C  TIME REQUIRED FOR THE SEMIANALYTICAL SOLUTION
C      
      TIME = (TAU*KR2*B2)/(RC*RC)
      TAUADD = TIME 
      FT2 = 0.0
C
C  MAIN DO-LOOP OVER NO. OF TIME STEPS
C
      DO 997 LPZ = 1, NTAU
      A = DLOG(2.0D0)/TIME
C
C   INITIALIZE ASUM ARRAY TO ZERO
C
      DO 30 I = 1, 20
         ASUM1(I) = 0.0
         ASUM2(I) = 0.0
30    CONTINUE
      FT1 = 0.0
C
C  MAIN DO-LOOP FOR THE SUMMATION OF FOURIER BACKTRANSFORM
C    ASSUMING THAT 25000 TERMS IS SUFFICIENT FOR CONVERGENCE.
C    EXPERIENCE HAS SHOWN THAT 25000 TERMS IS SUFFICIENT IN 
C    ESSENTIALLY ALL CASES
C
      DO 120 L = 1, 25000, NNMM
      I1 = L
      I2 = L + NNMM - 1
C
C  DO-LOOP OVER THE NO. OF STEHFEST WEIGHTING FACTORS (N)
C
      DO 90  J = 1, N
C
C  CALCULATION OF LAPLACE TRANSFORM VARIABLE
C
        P = DBLE(J)*A
C
C   CALL FSUM SUBROUTINE TO PERFORM SERIES SUMMATION BETWEEN TERMS
C       I1 AND I2 - ASUM ARRAY STORES CURRENT SUMMATION VALUE
C       FSUM CALCULATES OMEGA TERMS OF EQNS. (A14) AND (A18), AND
C       OMEGA SUBSCRIPT AND OMEGA STAR SUBSCRIPT TERMS OF BUTLER (1995)
C
cspe        CALL FSUM(I1, I2, ICN, OUT1, OUT2,IFLAG,IOBS)
        ASUM1(J) = ASUM1(J) + OUT1
        ASUM2(J) = ASUM2(J) + OUT2
        DINT1 = ASUM1(J)
        DINT2 = ASUM2(J)
C
C   CALCULATE EQN. (A14)- CONFINED CASE OR EQN. (A18)- UNCONFINED CASE
C     RESPECTIVE EQNS. FROM BUTLER (1995) ARE (13) AND (22)
C
        COEF = GAMMA2*HO*DINT2/(GAMMA2*P*DINT1+1.0)
        FP(J) = COEF
   90 CONTINUE
C
C   CALL STEFST SUBROUTINE TO PERFORM LAPLACE INVERSION USING THE
C      STEHFEST ALGORITHM
C
      CALL STEFST(FP,N,TIME,FT)
C
C  CHECK TO SEE IF SOLUTION HAS CONVERGED TO WITHIN RELATIVE ERROR (RERR).
C    IF SO, EXIT LOOP FOR FOURIER BACKTRANSFORM.
C
      ERR = DABS((FT-FT1)/FT)
      IF(ERR .LE. RERR) GO TO 150
      FT1 = FT
120   CONTINUE
      WRITE(12, *) ' SOLUTION DID NOT CONVERGE'
150   CONTINUE
C
C  PRINT RESULTS FOR CURRENT TIME
C                       
      TIMFAC = (KR2*B2)/(RC*RC)
      WRITE (*,105) TIME/TIMFAC, FT
      IF (FT .GT. 0.0) WRITE (8,106) TIME, FT/HO
      IF (FT .GT. 0.0) WRITE (9,106) TIME/TIMFAC, FT
  105 FORMAT (' FOR TIME = ',E13.6,'  THE HEAD = ',E13.6)
  106 FORMAT(2E13.6)
C
C  IF NORMALIZED HEAD LESS THAN USER-SPECIFIED VALUE AND HEADS ARE
C  DECREASING, STOP CALCULATION
C
      IF (FT/HO .LT. HTOL.AND.(FT2/FT).GT.1.D0) GO TO 9999
C
C  COMPUTE TIME FOR NEXT HEAD CALCULATION
C
      TAUADD = TAUADD * TAUINC
      TIME = TIME + TAUADD        
      FT2 = FT
997   CONTINUE
9999  CONTINUE
      STOP
      END
C***************************************************************************
C   STEHFEST SUBROUTINE
C***************************************************************************
cspe  This routine is called from Visual Basic code in GraphicKGS.xls.  
cspe  Attribute statements were added to access the subroutine in the compiled dll.
cspe  No other changes were made to the subroutine.

      SUBROUTINE STEFST(FP,N,TIME,FT)
!ms$attributes dllexport :: STEFST
!ms$attributes alias: 'STEFST' :: STEFST
      IMPLICIT DOUBLE PRECISION (A-H, O-Z)
      DOUBLE PRECISION FP(20),V(6,20),FT,TIME,A
C      
C  SUBROUTINE TO CALCULATE THE BACKTRANSFORM OF THE LAPLACE SPACE
C  SOLUTION USING THE STEHFEST ALGORITHM.  
C  FP IS THE DATA ARRAY TO BE BACKTRANSFORMED, HAVE N VALUES IN 
C  LAPLACE SPACE AT INTERVALS OF THE STEHFEST INCREMENT, LOG(2)/TIME = A
C  FT IS THE VALUE OF THE TRANSFORMED FUNCTION
C
C  DATA STATEMENTS GIVING STEHFEST WEIGHTING FACTORS FOR DIFFERENT
C   VALUES OF N
C
10    DATA (V(1,I),I=1,10)/
     + .8333333333333333D-01,-.3208333333333334D+02,.127900000000D+04,
     +-.1562366666666667D+05,.8424416666666666D+05,-.236957500000D+06,
     + .3759116666666667D+06,-.3400716666666667D+06,.164062500000D+06,
     +-.3281250000000000D+05/
12    DATA (V(2,I),I=1,12)/
     +-.1666666666666667D-01,.1601666666666667D+02,-.124700000000D+04,
     + .2755433333333333D+05,-.2632808333333333D+06,.132413870000D+07,
     +-.3891705533333333D+07,.7053286333333333D+07,-.800533650000D+07,
     + .5552830500000000D+07,-.2155507200000000D+07,.359251200000D+06/
14    DATA (V(3,I),I=1,14)/
     + .2777777777777778D-02,-.6402777777777779D+01, .92405000000D+03,
     +-.345979277777777D+05,.540321111111111D+06,-.439834636666666D+07,
     + .2108759177777778D+08,-.6394491304444444D+08, .12759757955D+09,
     +-.1701371880833333D+09, .1503274670333333D+09,-.84592161500D+08,
     + .2747888476666667D+08,-.3925554966666667D+07/
16    DATA (V(4,I),I=1,16)/
     +-.396825396825396D-03,.213373015873015D+01,-.551016666666666D+03,
     + .335001611111111D+05,-.812665111111111D+06,.100761837666666D+08,
     +-.732413829777777D+08,.339059632073015D+09,-.105253953627857D+10,
     + .225901332858333D+10,-.339970198443333D+10,.358245046170000D+10,
     +-.259149408136666D+10,.122704982876666D+10,-.342734555428571D+09,
     + .4284181942857143D+08/
18    DATA (V(5,I),I=1,18)/
     + .496031746031746D-04,-.609573412698412D+00,.274594047619047D+03,
     +-.263069567460317D+05,.957257201388888D+06,-.173586948458333D+08,
     + .182421222647222D+09,-.121853328830912D+10,.549168002528303D+10,
     +-.173621311152068D+11,.394550969035273D+11,-.652665169851750D+11,
     + .787300683282208D+11,-.685564441961208D+11,.419843434750535D+11,
     +-.171609347118392D+11,.420455003910267D+10,-.467172226566964D+09/
20    DATA (V(6,I),I=1,20)/
     +-.551146384479717D-05,.152386463844797D+00,-.117465476190476D+03,
     + .173424493386243D+05,-.922806928902116D+06,.237740877871031D+08,
     +-.349421166195370D+09,.324136985223187D+10,-.202769483072378D+11,
     + .894648298237972D+11,-.287020921147102D+12,.682992010281511D+12,
     +-.121908233005437D+13,.163757380084201D+13,-.164717748683611D+13,
     + .122192455444422D+13,-.648806558817532D+12,.233316653213705D+12,
     +-.5091380070546737D+11,   .5091380070546738D+10/
C
      A = DLOG(2.0D0)/TIME
      FT = 0.0D0
      J = (N -8)/2
C
C   DO LOOP OVER NO. OF WEIGHTING FACTORS (N)
C
      DO 40 I = 1, N
      FT = FT + V(J,I)*FP(I)
   40 CONTINUE
      FT = A*FT
      RETURN
      END
C**********************************************************************
C
C              FSUM SUBROUTINE
C     SUBROUTINE TO PERFORM THE FOURIER INVERSION SUMMATION.      
C
C**********************************************************************
C
C     HOW TO USE THIS SUBROUTINE:
C
C     1.  INSERT THE FOLLOWING COMMAND INTO THE CALLING PROGRAM:
C
C               CALL FSUM(M1,M2,ICN,OUT1,OUT2,IFLAG,IOBS)
C          WHERE
C             M1 FIRST TERM OF SUMMATION (INPUT)
C             M2 LAST TERM OF SUMMATION (INPUT)
C            ICN SET TO ZERO IF SCREEN IS AT THE CENTER OF A CONFINED 
C                  AQUIFER (INPUT)
C           OUT1 VALUE OF INTEGRATION OF F (OUTPUT)
C           OUT2 VALUE OF INTEGRATION OF G (OUTPUT)
C          IFLAG INDICATOR FOR CONFINED (=1) OR UNCONFINED (=2) CASE (INPUT)
C           IOBS INDICATOR FOR COMPUTATION OF HEADS AT THE TEST WELL (=0) OR 
C                  AT AN OBSERVATION WELL (INPUT)
C            
C
C     2.  SUPPLY AN EXTERNAL FUNCTION NAMED "F" AS THE FUNCTION BEING
C         SUMMED - "F" FUNCTION USED TO CALCULATE HEAD AT TEST WELL.
C
C     3.  SUPPLY AN EXTERNAL FUNCTION NAMED "G" AS THE FUNCTION BEING
C         SUMMED - "G" FUNCTION USED TO CALCULATE HEAD AT OBS. WELL.
C
C***********************************************************************
C
C      NOTE:
C
C        1. ANY PARAMETER NEEDED TO CALCULATE THE TERM INSIDE THE SUMMATION 
C           MUST BE PUT IN A COMMON BLOCK USED BY THE EXTERNAL FUNCTIONS 
C           "F" AND "G".
C
C**********************************************************************
C
cspe  The declaration statement of this subroutine was modified to include all variables within 
cspe  COMMON statements in the F and G functions.
cspe  Attribute statements were added so that this function could be called from the compiled dll.
      SUBROUTINE FSUM(M1, M2, ICN, VALUE1,VALUE2,RS,RC,RNK,
     *S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,X1,X2,PI2,ISKIN,IFLAG,R1,R2,
     *BETA,IOBS,ZD,Z1,Z2)
!ms$attributes dllexport :: FSUM
!ms$attributes alias: 'FSUM' :: FSUM
      IMPLICIT REAL*8 (A-H,O-Z)
      EXTERNAL F,G
C
C     CALCULATE THE TERM FOR N = 0 OF SUMMATION - THIS IS THE FIRST
C      TERM OF EQN. (B6) - (17) IS THE COMPARABLE EQN. FROM BUTLER (1995)
C
      SUMM1 = 0.0
      SUMM2 = 0.0
      IF (M1 .EQ. 1) THEN 
        SUMM1 = F(0,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,X1,X2,
     *PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
        IF (IOBS.EQ.1) THEN 
           SUMM2 = G(0,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,X1,
     *X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
        ENDIF 
      ENDIF  
C
C DO-LOOP 600 IS FOR SUMMATION FROM N=M1+ISTART-1 TO M2 -
C   IF SCREEN IS AT THE CENTER OF A CONFINED AQUIFER LOOPING PROCEEDS
C   IN INCREMENTS OF TWO - THIS IS BECAUSE ALL ODD NUMBERED STEPS
C   WILL BE ZERO AS A RESULT OF THE COS (IR*PI/2) TERM THAT ARISES WHEN
C   SCREEN IS AT THE CENTER OF A CONFINED AQUIFER - NOTE THAT M1 WILL
C   ALWAYS BE AN ODD NUMBER BECAUSE OF THE MANNER IN WHICH NNMM
C   IS CALCULATED.  IF UNCONFINED CASE, ONLY EVALUATE F AT ODD-
C   NUMBERED STEPS BECAUSE OF NATURE OF MODIFIED SINE TRANSFORM
C   USED IN UNCONFINED CASE.
C
      ISTART = 1
      ISTEP = 1
      IF(IFLAG.EQ.2)ISTEP=2
      IF (ICN .EQ. 0) ISTEP = 2
      IF (ICN .EQ. 0 .AND. IFLAG .EQ. 1) ISTART = 2
      DO 600 IR=M1 + ISTART - 1, M2, ISTEP
         NN = IR
         ADD1 = F(NN,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,X1,
     *X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
         SUMM1 = SUMM1 + ADD1
         IF (IOBS .EQ. 1) THEN
            ADD2 = G(NN,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,X1,
     *X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
            SUMM2 = SUMM2 + ADD2
         ENDIF 
600   CONTINUE
      VALUE1 = SUMM1
      IF (IOBS .EQ. 0) THEN
         VALUE2 = VALUE1
       ELSE
         VALUE2 = SUMM2
      ENDIF       
999   RETURN
      END
C*******************************************************************
C           FUNCTION F - FUNCTION TO BE SUMMED
C
C*******************************************************************
cspe  The declaration statement of this subroutine was modified to include all variables within 
cspe  COMMON statements.  The COMMON statements were commented out.
      DOUBLE PRECISION FUNCTION F(NQ,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,
     *P,AST1,AST2,X1,X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION MMBSK0, MMBSK1, MMBSI0, MMBSI1
cspe      COMMON /CMPPSLUG/ RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,
cspe     *                  X1,X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2
C
C  CALCULATE THE NU TERMS OF EQN. (A11)
C
      QN = DBLE(NQ)
      Q21 = R1*P
      Q22 = R2*P
      V1 = DSQRT(AST1*(QN**2.0) + Q21)
      V2 = DSQRT(AST2*(QN**2.0) + Q22)
      AKR = T1*V1/(T2*V2)
C
C  PERFORM CALCULATION FOR NO-SKIN CASE - SKIN CASE WILL REDUCE TO THIS
C    FOR CASE OF SKIN AND AQUIFER PROPERTIES THE SAME - HOWEVER, THE 
C    PROCESSING TIME IS CONSIDERABLY LONGER IF SKIN CALCULATIONS PERFORMED
C    FOR THE NO-SKIN CASE
C
      IF(ISKIN.EQ.0) THEN
         IF(NQ.EQ.0) THEN
           IF(IFLAG.EQ.1)THEN
             F=(1.0/BETA)*MMBSK0(2,V1,IER)/(V1*MMBSK1(2,V1,IER))
           ELSE
             F=0.0d0
           ENDIF
           GOTO 50
         ENDIF
         W = PI2 * QN*QN
         IF(IFLAG.EQ.1)THEN
           SINCOS = 8.0*((DSIN(QN*X1)*DCOS(QN*X2))**2)
         ELSE
           SINCOS = 32.0*((DSIN(QN*X1)*DSin(QN*X2))**2)
         ENDIF
         F = SINCOS*BETA*MMBSK0(2,V1,IER)/(W*V1*MMBSK1(2,V1,IER))            
C
C  USE EXTERNAL FUNCTIONS TO CALCULATE THE MODIFIED BESSEL FUNCTIONS -
C    NOTE THAT THE DELTA1 TERM USED HERE IS DEFINED AS DELTA2 IN EQN. (A12)
C    NOTE THAT ALL MODIFIED BESSEL FUNCTIONS OF THE FIRST KIND (In) ARE
C    EVALUATED AS EXP(-ARG)I(ARG) IN ORDER TO PREVENT MACHINE OVERFLOW
C    WHEN FUNCTIONS ARE EVALUATED.  SIMILARLY, ALL MODIFIED BESSEL FUNCTIONS
C    OF THE SECOND KIND (Kn) ARE EVALUATED AS EXP(ARG)K(ARG) IN ORDER
C    TO PREVENT MACHINE UNDERFLOW WHEN FUNCTIONS ARE EVALUATED.  THIS GIVES
C    RISE TO THE SZ TERM IN THE DNUMR AND DENOM EXPRESSIONS.
C
      ELSE
      DELTA1 = MMBSI0(2,V1*RNK,IER) * MMBSK1(2,V2*RNK,IER) + AKR *
     *         MMBSI1(2,V1*RNK,IER) * MMBSK0(2,V2*RNK,IER)
      DELTA2 = MMBSK0(2,V1*RNK,IER) * MMBSK1(2,V2*RNK,IER) - AKR *
     *         MMBSK0(2,V2*RNK,IER) * MMBSK1(2,V1*RNK,IER)
      SZ = DEXP(2.0 * V1 * (1.0 - RNK))
C
C  CALCULATE THE NUMERATOR AND DENOMINATOR OF THE f(N) TERM OF
C     EQNS. (B6) AND (B7)
C
      DNUMR = DELTA1*MMBSK0(2,V1,IER) - SZ*DELTA2*MMBSI0(2,V1,IER)
      DENOM = DELTA1*MMBSK1(2,V1,IER) + SZ*DELTA2*MMBSI1(2,V1,IER)
      IF (NQ .EQ. 0) THEN
C
C  IF NQ=0, CALCULATE FIRST TERM OF EQN. (B6)
C
      IF(IFLAG.EQ.1)THEN
        F=(1.0/BETA)*DNUMR/(V1*DENOM)
      ELSE
        F=0.0d0
      ENDIF
      GO TO 50
      ENDIF
C
C  IF NQ > 0, CALCULATE TERM OF INFINITE SERIES OF EQNS. (B6) AND (B7)
C
      W = PI2 * QN*QN
      IF(IFLAG.EQ.1)THEN
        SINCOS=8.0*((DSIN(QN*X1)*DCOS(QN*X2))**2)
      ELSE
        SINCOS=32.0*((DSIN(QN*X1)*DSin(QN*X2))**2) 
      ENDIF
C
C  CALCULATE THE VALUE OF THE FUNCTION BEING SUMMED
C
      F=SINCOS*BETA*DNUMR/(V1*W*DENOM)
      ENDIF
   50 CONTINUE
      RETURN
      END
C*******************************************************************
C           FUNCTION G - FUNCTION TO BE SUMMED
C
C*******************************************************************
cspe  The declaration statement of this subroutine was modified to include all variables within 
cspe  COMMON statements.  The COMMON statements were commented out.
      DOUBLE PRECISION FUNCTION G(NQ,RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,
     *P,AST1,AST2,X1,X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DOUBLE PRECISION MMBSK0, MMBSK1, MMBSI0, MMBSI1
cspe      COMMON /CMPPSLUG/ RS,RC,RNK,S1,S2,T1,T2,HO,R,B,B2,P,AST1,AST2,
cspe     *                  X1,X2,PI2,ISKIN,IFLAG,R1,R2,BETA,IOBS,ZD,Z1,Z2
C
C  CALCULATE THE NU TERMS OF EQN. (A11)
C
      QN = DBLE(NQ)
      Q21 = R1*P
      Q22 = R2*P
      V1 = DSQRT(AST1*(QN**2.0) + Q21)
      V2 = DSQRT(AST2*(QN**2.0) + Q22)
      AKR = T1*V1/(T2*V2)
C
C  PERFORM CALCULATION FOR NO-SKIN CASE - SKIN CASE WILL REDUCE TO THIS
C    FOR CASE OF SKIN AND AQUIFER PROPERTIES THE SAME - HOWEVER, THE 
C    PROCESSING TIME IS CONSIDERABLY LONGER IF SKIN CALCULATIONS PERFORMED
C    FOR THE NO-SKIN CASE
C
      IF(ISKIN.EQ.0) THEN 
         SS = DEXP(V1*(1.0-R))
         IF(NQ.EQ.0) THEN
           IF(IFLAG.EQ.1)THEN               
C
C   N=0 TERM OF EQN. (18) OF BUTLER (1995)
C           
             G=(1.0/BETA)*SS*MMBSK0(2,V1*R,IER)/(V1*
     C         MMBSK1(2,V1,IER))
           ELSE
             G=0.0d0
           ENDIF
           GOTO 50
         ENDIF
         W = PI2 * QN*QN * ZD
         IF(IFLAG.EQ.1)THEN
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF EQN. (18) OF BUTLER (1995)
C
          SINCOS = 8.0*(DSIN(QN*X1)*DCOS(QN*X2)*DSIN(QN*Z1)*DCOS(QN*Z2))
         ELSE
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF UNCONFINED ANALOGUE TO EQN. (18)
C         OF BUTLER (1995)
C
           SINCOS = 32.0*(DSIN(QN*X1)*DSin(QN*X2)*DSIN(QN*Z1)*
     *       DSIN(QN*Z2))      
         ENDIF
C
C    CALCULATION OF ONE ELEMENT OF SUMMATION OF (18) OF BUTLER (1995)
C          
         G = SINCOS*BETA*SS*MMBSK0(2,V1*R,IER)/(W*V1*MMBSK1(2,V1,IER))            
C
C  USE EXTERNAL FUNCTIONS TO CALCULATE THE MODIFIED BESSEL FUNCTIONS -
C    NOTE THAT THE DELTA1 TERM USED HERE IS DEFINED AS DELTA2 IN BUTLER (1995)
C    NOTE THAT ALL MODIFIED BESSEL FUNCTIONS OF THE FIRST KIND (In) ARE
C    EVALUATED AS EXP(-ARG)I(ARG) IN ORDER TO PREVENT MACHINE OVERFLOW
C    WHEN FUNCTIONS ARE EVALUATED.  SIMILARLY, ALL MODIFIED BESSEL FUNCTIONS
C    OF THE SECOND KIND (Kn) ARE EVALUATED AS EXP(ARG)K(ARG) IN ORDER
C    TO PREVENT MACHINE UNDERFLOW WHEN FUNCTIONS ARE EVALUATED.  THIS GIVES
C    RISE TO THE SZ TERMS IN THE DNUMR AND DENOM EXPRESSIONS.
C
      ELSE
      DELTA1 = MMBSI0(2,V1*RNK,IER) * MMBSK1(2,V2*RNK,IER) + AKR *
     *         MMBSI1(2,V1*RNK,IER) * MMBSK0(2,V2*RNK,IER)
      DELTA2 = MMBSK0(2,V1*RNK,IER) * MMBSK1(2,V2*RNK,IER) - AKR *
     *         MMBSK0(2,V2*RNK,IER) * MMBSK1(2,V1*RNK,IER)
C     
C   CASE OF OBS. WELL LYING RADIALLY OUTWARD OF SKIN RADIUS
C
      IF(R.GT.RNK) THEN
C
C  CALCULATE THE NUMERATOR AND DENOMINATOR OF THE f3(N) TERM OF
C     EQNS. (12) AND (21) OF BUTLER (1995) 
C
      SZ1 = DEXP(V2 * (R - RNK))
      SZ2 = DEXP(V1 * (1.0 - RNK))*SZ1 
      SZ3 = DEXP(V1 * (RNK - 1.0))*SZ1
      DNUMR = MMBSK0(2,V2*R,IER)*(DELTA1*MMBSK0(2,V1*RNK,IER)
     C  - DELTA2*MMBSI0(2,V1*RNK,IER))
      DENOM = MMBSK0(2,V2*RNK,IER)*(SZ3*DELTA1*MMBSK1(2,V1,IER) + 
     C    SZ2*DELTA2*MMBSI1(2,V1,IER))
      IF (NQ .EQ. 0) THEN
C
C  IF NQ=0, CALCULATE FIRST TERM OF EQN. (17) OF BUTLER (1995)
C
      IF(IFLAG.EQ.1)THEN
        G=(1.0/BETA)*DNUMR/(V1*DENOM)
      ELSE
        G=0.0d0
      ENDIF
      GO TO 50
      ENDIF
C
C  IF NQ > 0, CALCULATE TERM OF INFINITE SERIES OF EQNS. (17) AND (25) OF
C     BUTLER (1995)
C
      W = PI2 * QN*QN * ZD
      IF(IFLAG.EQ.1)THEN
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF EQN. (17) OF BUTLER (1995)
C      
        SINCOS=8.0*(DSIN(QN*X1)*DCOS(QN*X2)*DSIN(QN*Z1)*DCOS(QN*Z2))
      ELSE
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF EQN. (25) OF BUTLER (1995)
C      
        SINCOS=32.0*(DSIN(QN*X1)*DSin(QN*X2)*DSIN(QN*Z1)*DSIN(QN*Z2)) 
      ENDIF
C
C  CALCULATE THE VALUE OF ONE TERM OF THE FUNCTION BEING SUMMED IN 
C     EQNS. (17) AND (25) OF BUTLER (1995)
C
      G=SINCOS*BETA*DNUMR/(V1*W*DENOM)
      ELSE
C                
C  CASE OF OBS. WELL BETWEEN SCREEN RADIUS AND SKIN RADIUS
C
C  CALCULATE THE NUMERATOR AND DENOMINATOR OF THE f2(N) TERM OF
C     EQNS. (12) AND (21) OF BUTLER (1995)
C
      SZ1 = DEXP(2.0 * V1 * (R - RNK))
      SZ2 = DEXP(-2.0 * V1 * RNK + V1 * (1.0 + RNK)) 
      SZ3 = DEXP(V1 * (R - 1.0))
      DNUMR = DELTA1*MMBSK0(2,V1*R,IER) - SZ1*DELTA2*MMBSI0(2,V1*R,IER)
      DENOM = SZ3*DELTA1*MMBSK1(2,V1,IER) + SZ2*DELTA2*MMBSI1(2,V1,IER)
      IF (NQ .EQ. 0) THEN
C
C  IF NQ=0, CALCULATE FIRST TERM OF EQN. (17) OF BUTLER (1995)
C
      IF(IFLAG.EQ.1)THEN
        G=(1.0/BETA)*DNUMR/(V1*DENOM)
      ELSE
        G=0.0d0
      ENDIF
      GO TO 50
      ENDIF
C
C  IF NQ > 0, CALCULATE TERM OF INFINITE SERIES OF EQNS. (17) AND
C    (25) OF BUTLER (1995)
C
      W = PI2 * QN*QN * ZD
      IF(IFLAG.EQ.1)THEN  
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF EQN. (17) OF BUTLER (1995)
C      
        SINCOS=8.0*(DSIN(QN*X1)*DCOS(QN*X2)*DSIN(QN*Z1)*DCOS(QN*Z2))
      ELSE
C
C   TRIG. FUNCTIONS OF N > 0 TERMS OF EQN. (25) OF BUTLER (1995)
C      
        SINCOS=32.0*(DSIN(QN*X1)*DSin(QN*X2)*DSIN(QN*Z1)*DSIN(QN*Z2)) 
      ENDIF
C
C  CALCULATE THE VALUE OF ONE TERM OF THE FUNCTION BEING SUMMED IN 
C     EQNS. (17) AND (25) OF BUTLER (1995)
C
      G=SINCOS*BETA*DNUMR/(V1*W*DENOM)
      ENDIF
      ENDIF
   50 CONTINUE
      RETURN
      END